@@ -28,6 +28,7 @@ extern crate http_bytes;
 use http_bytes::http;
 
 use std::net::{IpAddr, SocketAddr};
+use std::path::PathBuf;
 
 use structopt::StructOpt;
 
@@ -364,7 +365,7 @@ struct Opt {
 
     #[structopt(
         long = "socks5",
-        help = "Use specified address:port as a SOCKS5 proxy. Note that proxy authentication is not supported yet. Example: --socks5 127.0.0.1:9050"
+        help = "Use specified address:port as a SOCKS5 proxy. Example: --socks5 127.0.0.1:9050"
     )]
     auto_socks5: Option<SocketAddr>,
 
@@ -375,6 +376,12 @@ struct Opt {
     )]
     socks5_bind_script: Option<OsString>,
 
+    #[structopt(
+        long = "socks5-user-pass",
+        help = "[A] Specify username:password for SOCKS5 proxy. If not specified, the default is to use no authentication."
+    )]
+    socks5_user_pass: Option<String>,
+
     #[structopt(
         long = "tls-domain",
         alias = "ssl-domain",
@@ -522,10 +529,15 @@ struct Opt {
     #[structopt(long = "--no-async-stdio")]
     pub noasyncstdio: bool,
 
-    /// Add `Authorization: Basic` HTTP request header with this base64-encoded parameter
+    /// Add `Authorization: Basic` HTTP request header with this base64-encoded parameter.
+    /// Also available as `WEBSOCAT_BASIC_AUTH` environment variable
     #[structopt(long = "--basic-auth")]
     pub basic_auth: Option<String>,
 
+    /// Add `Authorization: Basic` HTTP request header base64-encoded content of the specified file
+    #[structopt(long = "--basic-auth-file")]
+    pub basic_auth_file: Option<PathBuf>,
+
     /// [A] Wait for reading to finish before closing foreachmsg:'s peer
     #[structopt(long = "--foreachmsg-wait-read")]
     pub foreachmsg_wait_reads: bool,
@@ -601,50 +613,6 @@ struct Opt {
     #[structopt(long = "uncompress-gzip")]
     pub uncompress_gzip: bool,
 
-    /// [A] Load specified symbol from specified native library and use it for `native_plugin_transform_a`.
-    /// Format is `symbol@library_file`. If `symbol@` is omitted, `websocat_transform` is implied.
-    #[cfg(feature = "native_plugins")]
-    #[structopt(long = "native-plugin-a",  parse(try_from_str = "websocat::transform_peer::load_symbol"))]
-    pub native_transform_a: Option<websocat::transform_peer::Sym>,
-
-    /// [A] Load specified symbol from specified native library and use it for `native_plugin_transform_b`.
-    #[cfg(feature = "native_plugins")]
-    #[structopt(long = "native-plugin-b", parse(try_from_str = "websocat::transform_peer::load_symbol"))]
-    pub native_transform_b: Option<websocat::transform_peer::Sym>,
-
-    /// [A] Load specified symbol from specified native library and use it for `native_plugin_transform_c`.
-    #[cfg(feature = "native_plugins")]
-    #[structopt(long = "native-plugin-c", parse(try_from_str = "websocat::transform_peer::load_symbol"))]
-    pub native_transform_c: Option<websocat::transform_peer::Sym>,
-
-    /// [A] Load specified symbol from specified native library and use it for `native_plugin_transform_d`.
-    #[cfg(feature = "native_plugins")]
-    #[structopt(long = "native-plugin-d", parse(try_from_str = "websocat::transform_peer::load_symbol"))]
-    pub native_transform_d: Option<websocat::transform_peer::Sym>,
-
-    /// [A] Load specified symbol from specified wasm module and use it for `wasm_plugin_transform_a:`.
-    /// Format is `symbol@library_file`. If `symbol@` is omitted, `websocat_transform` is implied.
-    /// The wasm module should also have `malloc` and `free` functions exposed.
-    /// Prepend `library_file` with `!` to load serialized cwasm produced by `wasmtime compile` instead of compining the module in Websocat.
-    #[cfg(feature = "wasm_plugins")]
-    #[structopt(long = "wasm-plugin-a",  parse(try_from_str = "websocat::wasm_transform_peer::load_symbol"))]
-    pub wasm_transform_a: Option<websocat::wasm_transform_peer::Handle>,
-
-    /// [A] Load specified symbol from specified wasm module and use it for `wasm_plugin_transform_b:`.
-    #[cfg(feature = "wasm_plugins")]
-    #[structopt(long = "wasm-plugin-b",  parse(try_from_str = "websocat::wasm_transform_peer::load_symbol"))]
-    pub wasm_transform_b: Option<websocat::wasm_transform_peer::Handle>,
-
-    /// [A] Load specified symbol from specified wasm module and use it for `wasm_plugin_transform_c:`.
-    #[cfg(feature = "wasm_plugins")]
-    #[structopt(long = "wasm-plugin-c",  parse(try_from_str = "websocat::wasm_transform_peer::load_symbol"))]
-    pub wasm_transform_c: Option<websocat::wasm_transform_peer::Handle>,
-
-    /// [A] Load specified symbol from specified wasm module and use it for `wasm_plugin_transform_d:`.
-    #[cfg(feature = "wasm_plugins")]
-    #[structopt(long = "wasm-plugin-d",  parse(try_from_str = "websocat::wasm_transform_peer::load_symbol"))]
-    pub wasm_transform_d: Option<websocat::wasm_transform_peer::Handle>,
-
     /// [A] Omit `jsonrpc` field when using `--jsonrpc`, e.g. for Chromium
     #[structopt(long = "jsonrpc-omit-jsonrpc")]
     pub jsonrpc_omit_jsonrpc: bool,
@@ -672,6 +640,10 @@ struct Opt {
     /// [A] Only affect one direction of the `lengthprefixed:` overlay, bypass tranformation for the other one.
     #[structopt(long = "--lengthprefixed-skip-write-direction")]
     pub lengthprefixed_skip_write_direction: bool,
+
+    /// Set `User-Agent` request header to this value. Similar to setting it with `-H`.
+    #[structopt(long = "--ua")]
+    pub useragent: Option<String>,
 }
 
 // TODO: make it byte-oriented/OsStr?
@@ -920,6 +892,7 @@ fn run() -> Result<()> {
             socks_destination
             auto_socks5
             socks5_bind_script
+            socks5_user_pass
             tls_domain
             max_parallel_conns
             ws_ping_interval
@@ -985,27 +958,33 @@ fn run() -> Result<()> {
                 prometheus
             }
         }
-        #[cfg(feature = "native_plugins")]
-        {
-            opts! {
-                native_transform_a
-                native_transform_b
-                native_transform_c
-                native_transform_d
-            }
-        }
-        #[cfg(feature = "wasm_plugins")]
-        {
-            opts! {
-                 wasm_transform_a
-                 wasm_transform_b
-                 wasm_transform_c
-                 wasm_transform_d
-            }
-        }
     };
 
+    if let Some(x) = cmd.useragent {
+        opts.custom_headers.push(("User-Agent".to_owned(), x.as_bytes().to_vec()));
+        opts.request_headers.push((http::header::USER_AGENT, http::header::HeaderValue::from_bytes(x.as_bytes()).unwrap()));
+    }
+
+    let mut basic_auth_content : Option<String> = None;
+
     if let Some(ba) = cmd.basic_auth {
+        basic_auth_content = Some(ba);
+    }
+    if let Ok(ba) = std::env::var("WEBSOCAT_BASIC_AUTH") {
+        if basic_auth_content.is_some() {
+            return Err("Multiple request basic auth options specified simultaneously")?;
+        }
+        basic_auth_content = Some(ba);
+    }
+    if let Some(baf) = cmd.basic_auth_file {
+        if basic_auth_content.is_some() {
+            return Err("Multiple request basic auth options specified simultaneously")?;
+        }
+        let x = std::fs::read_to_string(&baf).map_err(|e|{error!("Failed to read `{:?}`", baf); e})?;
+        basic_auth_content = Some(x.trim().to_owned());
+    }
+
+    if let Some(ba) = basic_auth_content {
         let x = base64::encode(&ba);
         let q = format!("Basic {}", x);
         opts.custom_headers.push(("Authorization".to_owned(), q.as_bytes().to_vec()));
