@@ -42,6 +42,20 @@ fn some_checks(s: &str) -> Result<()> {
         }
     }
 
+    #[cfg(not(feature = "crypto_peer"))]
+    {
+        if s.starts_with("crypto:") {
+            Err("`crypto:` support is not compiled in")?
+        }
+    }
+
+    #[cfg(not(feature = "prometheus_peer"))]
+    {
+        if s.starts_with("metrics:") || s.starts_with("prometheus:") {
+            Err("`prometheus:` support is not compiled in")?
+        }
+    }
+
     Ok(())
 }
 
